import { Pred, PredIndexedOptional, PredIndexed } from './_types';
/**
 * Returns the value of the first element in the array where predicate is true, and undefined otherwise.
 * @param items the array
 * @param fn the predicate
 * @signature
 *    R.find(items, fn)
 *    R.find.indexed(items, fn)
 * @example
 *    R.find([1, 3, 4, 6], n => n % 2 === 0) // => 4
 *    R.find.indexed([1, 3, 4, 6], (n, i) => n % 2 === 0) // => 4
 * @data_first
 * @indexed
 * @pipeable
 * @category Array
 */
export declare function find<T>(array: readonly T[], fn: Pred<T, boolean>): T | undefined;
/**
 * Returns the value of the first element in the array where predicate is true, and undefined otherwise.
 * @param fn the predicate
 * @signature
 *    R.find(fn)(items)
 *    R.find.indexed(fn)(items)
 * @example
 *    R.pipe(
 *      [1, 3, 4, 6],
 *      R.find(n => n % 2 === 0)
 *    ) // => 4
 *    R.pipe(
 *      [1, 3, 4, 6],
 *      R.find.indexed((n, i) => n % 2 === 0)
 *    ) // => 4
 * @data_last
 * @indexed
 * @pipeable
 * @category Array
 */
export declare function find<T = never>(fn: Pred<T, boolean>): (array: readonly T[]) => T | undefined;
export declare namespace find {
    function indexed<T>(array: readonly T[], fn: PredIndexed<T, boolean>): T | undefined;
    function indexed<T>(fn: PredIndexed<T, boolean>): (array: readonly T[]) => T | undefined;
    const lazy: (<T>(fn: PredIndexedOptional<T, boolean>) => (value: T, index?: number | undefined, array?: T[] | undefined) => {
        done: boolean;
        hasNext: boolean;
        next: T;
    }) & {
        single: true;
    };
    const lazyIndexed: (<T>(fn: PredIndexedOptional<T, boolean>) => (value: T, index?: number | undefined, array?: T[] | undefined) => {
        done: boolean;
        hasNext: boolean;
        next: T;
    }) & {
        indexed: true;
    } & {
        single: true;
    };
}
//# sourceMappingURL=find.d.ts.map