import { PredIndexed } from './_types';
/**
 * Returns the max element using the provided predicate.
 * @param fn the predicate
 * @signature
 *    R.maxBy(fn)(array)
 *    R.maxBy.indexed(fn)(array)
 * @example
 *    R.pipe(
 *      [{a: 5}, {a: 1}, {a: 3}],
 *      R.maxBy(x => x.a)
 *    ) // { a: 5 }
 * @data_last
 * @indexed
 * @category Array
 */
export declare function maxBy<T>(fn: (item: T) => number): (items: readonly T[]) => T | undefined;
/**
 * Returns the max element using the provided predicate.
 * @param items the array
 * @param fn the predicate
 * @signature
 *    R.maxBy(array, fn)
 *    R.maxBy.indexed(array, fn)
 * @example
 *    R.maxBy(
 *      [{a: 5}, {a: 1}, {a: 3}],
 *      x => x.a
 *    ) // { a: 5 }
 * @data_first
 * @indexed
 * @category Array
 */
export declare function maxBy<T>(items: readonly T[], fn: (item: T) => number): T | undefined;
export declare namespace maxBy {
    function indexed<T>(array: readonly T[], fn: PredIndexed<T, number>): T | undefined;
    function indexed<T>(fn: PredIndexed<T, number>): (array: readonly T[]) => T | undefined;
}
//# sourceMappingURL=maxBy.d.ts.map