import { PredIndexed } from './_types';
/**
 * Returns the min element using the provided predicate.
 * @param fn the predicate
 * @signature
 *    R.minBy(fn)(array)
 *    R.minBy.indexed(fn)(array)
 * @example
 *    R.pipe(
 *      [{a: 5}, {a: 1}, {a: 3}],
 *      R.minBy(x => x.a)
 *    ) // { a: 1 }
 * @data_last
 * @indexed
 * @category Array
 */
export declare function minBy<T>(fn: (item: T) => number): (items: readonly T[]) => T | undefined;
/**
 * Returns the min element using the provided predicate.
 * @param items the array
 * @param fn the predicate
 * @signature
 *    R.minBy(array, fn)
 *    R.minBy.indexed(array, fn)
 * @example
 *    R.minBy(
 *      [{a: 5}, {a: 1}, {a: 3}],
 *      x => x.a
 *    ) // { a: 1 }
 * @data_first
 * @indexed
 * @category Array
 */
export declare function minBy<T>(items: readonly T[], fn: (item: T) => number): T | undefined;
export declare namespace minBy {
    function indexed<T>(array: readonly T[], fn: PredIndexed<T, number>): T | undefined;
    function indexed<T>(fn: PredIndexed<T, number>): (array: readonly T[]) => T | undefined;
}
//# sourceMappingURL=minBy.d.ts.map